@@ -21,11 +21,6 @@
 #define USE_MMAP
 #endif
 
-/* Use memory alignment workaround or not */
-#ifdef __ia64__
-#define ALIGNMENT_WORKAROUND
-#endif
-
 /* Avoid unaligned memcpy on /dev/mem */
 #ifdef __aarch64__
 #define USE_SLOW_MEMCPY
